import pandas as pd
from jdatetime import date as jdate
import requests
import os
import io


def convert_to_shamsi(date):
    date = str(date)
    date_shamsi = jdate.fromgregorian(
        day=int(date[-2:]), month=int(date[4:6]), year=int(date[:4])
    ).strftime("%Y/%m/%d")
    return date_shamsi


def get_data(ticker):
    try:
        # open data file in the package
        tse_dir = os.path.dirname(__file__)
        dfm = pd.read_csv(f'{tse_dir}/data/TSE_Symbol_List.csv')
        # find the ticker ins code for generating the url
        code = dfm.loc[dfm['Symbol'] == ticker, 'Code'].iat[0]

        # create url for given ticker code and sending a request for data
        url = "http://www.tsetmc.com/tsev2/data/Export-txt.aspx?t=i&a=1&b=0&i={}"
        response = requests.get(url.format(code))
        file_object = io.StringIO(response.content.decode('utf-8'))
        # read downloaded data with pandas
        df = pd.read_csv(file_object)

        # changing columns' names and date formant from int to date
        df.columns = ['TICKER', 'DTYYYYMMDD', 'FIRST', 'HIGH', 'LOW', 'CLOSE',
                      'VALUE', 'VOL', 'OPENINT', 'PER', 'OPEN', 'LAST']
        df['Date'] = pd.to_datetime(df['DTYYYYMMDD'].astype(str), format='%Y%m%d')
        df["date_shamsi"] = df["DTYYYYMMDD"].apply(convert_to_shamsi)
        df.set_index('Date', inplace=True)
        df.drop(columns=["DTYYYYMMDD"], inplace=True)
        df.sort_index(inplace=True)

        return df

    except IndexError:
        return 'No match'



import pandas as pd
import matplotlib.pyplot as plt


def close_vol(df):
    ax1 = plt.subplot2grid((6,1), (0,0), rowspan=5, colspan=1)
    ax2 = plt.subplot2grid((6,1), (5,0), rowspan=1, colspan=1,sharex=ax1)

    ax1.plot(df.index, df['CLOSE'])
    ax2.bar(df.index, df['VOL'])

    return plt.show()


def ema(df, short, long):
    emasused = [short, long]
    # calculate exponential moving average for different periods and add them to your dataframe
    for x in emasused:
        ema = x
        df["Ema_"+str(ema)] = round(df.loc[:, ['CLOSE']].ewm(span=ema, adjust=False).mean(),2)

    # plot stock close price, short, and long periods with line chart.
    df.plot(kind='line',
            y=['CLOSE', f'Ema_{short}', f'Ema_{long}'],
            color=['black', 'b', 'r'],
            figsize=(16, 6))

    return plt.show()


def sma(df, short, long):
    smasused = [short, long]
    # # calculate simple moving average for different periods and add them to your dataframe
    for x in smasused:
        sma = x
        df["Sma_"+str(sma)] = round(df.loc[:, ['CLOSE']].rolling(window=sma, min_periods=0).mean(), 2)

    # plot stock close price, short, and long periods with line chart.
    df.plot(kind='line',
            y=['CLOSE', f'Sma_{short}', f'Sma_{long}'],
            color=['black', 'b', 'r'],
            figsize=(16, 6))

    return plt.show()


def return_dev(df):
    close_px = df['CLOSE']
    rets = close_px / close_px.shift(1) - 1
    rets.plot(label='return')
    return plt.show()


import requests
from bs4 import BeautifulSoup
import pandas as pd
import os


def update():
    """
    necessary data is attached into the package data file, so you can use them without
    running this function because it could take a few minute to update the main csv file.
    The main csv file contains symbol names and their code to generate url for downloading
    historical data. Over time this list on the tsetmc.com website would change and you can
    update your list with this function.
    """
    main_df = pd.DataFrame()
    # list of all pages that we should grab stock information from them and insert in our file
    pages = ['0', 'آ', 'ا', 'ب', 'پ', 'ت', 'ث', 'ج', 'چ', 'ح', 'خ', 'د', 'ذ', 'ر', 'ز', 'ژ', 'س', 'ش', 'ص', 'ض',
             'ط', 'ظ', 'ع', 'غ', 'ف', 'ق', 'ک', 'گ', 'ل', 'م', 'ن', 'و', 'ه', 'ی']

    # load all pages of symbols and insert data into dataframe
    for page in pages:

        url = f'http://www.tsetmc.com/Loader.aspx?ParTree=111C1416&firstChar={page}'
        r = requests.get(url)

        # web scraping with BeautifulSoup
        soup = BeautifulSoup(r.content, 'html.parser')
        # specify table in html page and table rows
        table = soup.find(id='tblToGrid')
        rows = table.find_all('tr')

        # find and insert each cell into a table
        table = []
        t = 0
        for tr in rows:
            td = tr.find_all('td')
            t += 1
            name = [i.text for i in td]
            table.append(name)

        # convert table to dataframe and insert headers
        df = pd.DataFrame(table)
        # take the data less the header row
        df = df[1:]
        df.columns = ['Symbol_Code', 'Group', 'Industries_Group', 'Table', 'En_Symbol', 'EN_name', 'Symbol1', 'Name1']

        # append each page of table to the main dataframe
        if main_df.empty:
            main_df = df
        else:
            main_df = main_df.append(df, ignore_index=True)

    # split name and symbol columns contents to separate columns
    new = main_df["Name1"].str.split(",", n=1, expand=True)
    main_df["Code"] = new[0]
    main_df["Name"] = new[1]

    new1 = main_df["Symbol1"].str.split(",", n=1, expand=True)
    main_df["a"] = new1[0]
    main_df["Symbol"] = new1[1]

    # drop redundant columns
    main_df.drop(columns=["Name1", 'a', "Symbol1"], inplace=True)

    # save main dataframe into CSV file
    tse_dir = os.path.dirname(__file__)
    path = f'{tse_dir}/data/TSE_Symbol_List.csv'
    main_df.to_csv(path, encoding='utf-8', index=False)

    print('Database has updated!')
    return main_df


def add(code, symbol):
    """
    tsetms website may update the symbol list with delay and if you want to load your new added stock
    with this package, you can simply add your specific symbol to the list with this function.
    """
    # finding file path in the package
    tse_dir = os.path.dirname(__file__)
    # open main csv file and add a row
    dfm = pd.read_csv(f'{tse_dir}/data/TSE_Symbol_List.csv')
    new_row = {'Code': code, 'Symbol': symbol}
    dfm = dfm.append(new_row, ignore_index=True)
    # replace new file with old file in the package
    path = f'{tse_dir}/data/TSE_Symbol_List.csv'
    dfm.to_csv(path, encoding='utf-8', index=False)

    return print('Symbol added to the list!')


from os import path
import setuptools

directory = path.abspath(path.dirname(__file__))

with open(path.join(directory, "README.md")) as f:
    long_description = f.read()

setuptools.setup(
    name="tse_dataloader",
    version="0.1.1",
    author="Aliik7",
    author_email="aliik2@gmail.com",
    description="Python package for downloading Tehran Stock Exchange data and analysing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aliik7/tse_dataloader",
    packages=setuptools.find_packages(),
    install_requires=["matplotlib", "pandas", "bs4", "requests", "jdatetime"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


